@@ -1,4 +1,4 @@
-# Copyright 2013-2018 Reahl Software Services (Pty) Ltd. All rights reserved.
+# Copyright 2013-2020 Reahl Software Services (Pty) Ltd. All rights reserved.
 #
 #    This file is part of Reahl.
 #
@@ -16,8 +16,6 @@
 
 # Copyright (C) 2006 Reahl Software Services (Pty) Ltd.  All rights reserved. (www.reahl.org)
 
-from __future__ import print_function, unicode_literals, absolute_import, division
-import os
 
 from selenium import webdriver
 from selenium.common.exceptions import WebDriverException
@@ -67,9 +65,9 @@ class WebServerFixture(Fixture):
 
     @property
     def web_driver(self):
-        return self.chrome_driver
+#        return self.chrome_driver
 #        return self.phantomjs_driver
-#        return self.firefox_driver
+        return self.firefox_driver
 
     def new_phantomjs_driver(self):
         driver = webdriver.PhantomJS() # or add to your PATH
@@ -87,13 +85,15 @@ class WebServerFixture(Fixture):
         # We're on 48.0 now on local machines, but on 31 on travis
 
         fp = FirefoxProfile()
-#        fp.set_preference("focusmanager.testmode", False)
-#        fp.set_preference('plugins.testmode', False)
-        fp.set_preference('webdriver_enable_native_events', True)
-        fp.set_preference('webdriver.enable.native.events', True)
-        fp.set_preference('enable.native.events', True)
-        fp.native_events_enabled = True
+##        fp.set_preference("focusmanager.testmode", False)
+##        fp.set_preference('plugins.testmode', False)
+#        fp.set_preference('webdriver_enable_native_events', True)
+#        fp.set_preference('webdriver.enable.native.events', True)
+#        fp.set_preference('enable.native.events', True)
+#        fp.native_events_enabled = True
 
+        fp.set_preference('webdriver.log.file', '/tmp/firefox.webdriver.log')
+        fp.set_preference('webdriver.firefox.logfile', '/tmp/firefox.log')
         fp.set_preference('network.http.max-connections-per-server', 1)
         fp.set_preference('network.http.max-persistent-connections-per-server', 0)
         fp.set_preference('network.http.spdy.enabled', False)
@@ -118,7 +118,7 @@ class WebServerFixture(Fixture):
         options = Options()
         options.add_argument('--disable-preconnect')
         options.add_argument('--dns-prefetch-disable')
-        options.add_argument('--start-maximized')
+#        options.add_argument('--start-maximized')  # This breaks xpra pair programming currently.
         options.add_argument('--no-sandbox')  # Needed to be able to run a user-installed version of chromium on travis
         options.binary_location = Executable('chromium-browser').executable_file  # To run a custom-installed chromium as picked up by the PATH
         #--enable-http-pipelining
@@ -150,7 +150,7 @@ class WebServerFixture(Fixture):
 
     def set_noop_app(self):
         # selenium.stop() hits the application its opened on again. NoopApp just ensures this does not break:
-        class NoopApp(object):
+        class NoopApp:
             def __call__(self, environ, start_response):
                 status = '403 Forbidden'
                 response_headers = [('Content-type','text/plain')]
@@ -172,12 +172,14 @@ class WebServerFixture(Fixture):
     def stop_servers(self):
         if self.is_instantiated('reahl_server'):
             self.reahl_server.set_noop_app() # selenium.stop() hits the application its opened on again.
-            self.reahl_server.restore_handlers()
         if self.is_instantiated('firefox_driver'):
+            self.firefox_driver.close()
             self.firefox_driver.quit()
         if self.is_instantiated('chrome_driver'):
+            self.chrome_driver.close()
             self.chrome_driver.quit()
         if self.is_instantiated('reahl_server'):
+            self.reahl_server.restore_handlers()
             self.reahl_server.stop()
 
     def new_test_dependencies(self):
